<?php

/**
 * Plugin Name: RodPay Lightning for WooCommerce
 * Plugin URI: https://rodpay.com/#wordpress_plugin
 * Description: Accept Bitcoin Lightning payments on your WooCommerce store with RodPay.
 * Requires Plugins: woocommerce
 * Version: 1.0.0
 * Author: rodpay2025
 * Author URI: https://rodpay.com
 * License: GPL2
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: rodpay-lightning-for-woocommerce
 */


 if (! defined("ABSPATH")) exit; // Exit if accessed directly

if (! in_array("woocommerce/woocommerce.php", apply_filters("active_plugins", get_option("active_plugins")))) {
    return;
}

add_action("plugins_loaded", "rodpay_lightning_for_woocommerce_init", 11);

function rodpay_lightning_for_woocommerce_init() {
    if (! class_exists("WC_Payment_Gateway")) {
        return;
    }

    class WC_Gateway_RodPay_Lightning extends WC_Payment_Gateway {
        public function __construct() {
            $this->id = "rodpay_lightning";
            $this->icon = apply_filters("woocommerce_rodpay_lightning_icon", plugins_url("assets/rodpay-lightning-icon.png", __FILE__));
            $this->has_fields = false;
            $this->method_title = __("RodPay Lightning", "rodpay-lightning-for-woocommerce");
            $this->method_description = __("Accept Bitcoin Lightning payments on your WooCommerce store with RodPay.", "rodpay-lightning-for-woocommerce");
            $this->supports = array("products");

            $this->init_form_fields();
            $this->init_settings();

            $this->title = $this->get_option("title");
            $this->description = $this->get_option("description");

            add_action("woocommerce_update_options_payment_gateways_" . $this->id, array($this, "process_admin_options"));

            add_action("woocommerce_api_rodpay_lightning_webhook", array($this, "webhook"));
        }

        public function init_form_fields() {
            $this->form_fields = array(
                "enabled" => array(
                    "title" => __("Enable/Disable", "rodpay-lightning-for-woocommerce"),
                    "type" => "checkbox",
                    "label" => __("Enable RodPay Lightning", "rodpay-lightning-for-woocommerce"),
                    "default" => "no",
                ),
                "title" => array(
                    "title" => __("Title", "rodpay-lightning-for-woocommerce"),
                    "type" => "text",
                    "description" => __("This controls the title which the user sees during checkout.", "rodpay-lightning-for-woocommerce"),
                    "default" => "Bitcoin Lightning",
                    "desc_tip" => true,
                ),
                "description" => array(
                    "title" => __("Description", "rodpay-lightning-for-woocommerce"),
                    "type" => "textarea",
                    "description" => __("This controls the description which the user sees during checkout.", "rodpay-lightning-for-woocommerce"),
                    "default" => "Pay with Bitcoin Lightning.",
                ),
                "instructions" => array(
                    "title" => __("Instructions", "rodpay-lightning-for-woocommerce"),
                    "type" => "textarea",
                    "description" => __("Instructions for the customer.", "rodpay-lightning-for-woocommerce"),
                    "default" => "Please scan QR code using your lightning wallet app to proceed payment.",
                ),
                "api_key" => array(
                    "title" => __("API Key", "rodpay-lightning-for-woocommerce"),
                    "type" => "text",
                    "description" => __("API Key from RodPay account. You can get API Key from <a href='https://app.rodpay.com/integration/' target='_blank'>here.</a>", "rodpay-lightning-for-woocommerce"),
                    "default" => "",
                ),
                "expiry" => array(
                    "title" => __("Invoice Expiry", "rodpay-lightning-for-woocommerce"),
                    "type" => "number",
                    "description" => __("Lightning invoice expiration time in minutes.", "rodpay-lightning-for-woocommerce"),
                    "default" => 2,
                ),
                "invoice_description" => array(
                    "title" => __("Invoice Description", "rodpay-lightning-for-woocommerce"),
                    "type" => "text",
                    "description" => __("Lightning invoice description template. `{{order_id}}` will be replaced with real Order ID.", "rodpay-lightning-for-woocommerce"),
                    "default" => "Invoice for Order - {{order_id}}",
                ),
                "invoice_currency" => array(
                    "title" => __("Invoice Currency", "rodpay-lightning-for-woocommerce"),
                    "type" => "text",
                    "description" => __("Lightning invoice amount currency.", "rodpay-lightning-for-woocommerce"),
                    "default" => "usd",
                    "required" => true,
                ),
            );
        }

        public function validate_title_field($key, $value) {
            if (empty($value)) {
                WC_Admin_Settings::add_error("Title is required.");
            }
            return $value;
        }

        public function validate_description_field($key, $value) {
            if (empty($value)) {
                WC_Admin_Settings::add_error("Description is required.");
            }
            return $value;
        }

        public function validate_invoice_currency_field($key, $value) {
            if (empty($value)) {
                WC_Admin_Settings::add_error("Invoice Currency is required.");
            }
            return $value;
        }

        public function validate_invoice_description_field($key, $value) {
            if (empty($value)) {
                WC_Admin_Settings::add_error("Invoice Description is required.");
            }
            return $value;
        }

        public function validate_expiry_field($key, $value) {
            if (empty($value)) {
                WC_Admin_Settings::add_error("Invoice Expiry is required.");
            }
            return $value;
        }

        public function validate_api_key_field($key, $value) {
            if (empty($value)) {
                WC_Admin_Settings::add_error("API Key is required.");
            }
            $response = wp_remote_get("https://api.rodpay.com/integration/woocommerce/verify/", array(
                "headers" => array(
                    "RodPay-Api-Key" => $value,
                ),
            ));

            if (is_wp_error($response) || wp_remote_retrieve_response_code($response) !== 200) {
                WC_Admin_Settings::add_error("Invalid API Key. Please use correct one.");
            }
            return $value;
        }

        public function get_form_fields() {
            return $this->form_fields;
        }
        public function get_form_fields_html() {
            $fields = $this->get_form_fields();
            $html = "";
            foreach ($fields as $key => $field) {
                $html .= "<tr>";
                $html .= "<th scope='row' class='titledesc'><label for='{$key}'>{$field['title']}</label></th>";
                $html .= "<td class='forminp'><fieldset><legend class='screen-reader-text'><span>{$field['title']}</span></legend>";
                $html .= "<textarea name='{$key}' id='{$key}' class='short' style='width: 50%;'>{$field['default']}</textarea>";
                $html .= "<p class='description'>{$field['description']}</p>";
                $html .= "</fieldset></td>";
                $html .= "</tr>";
            }
            return $html;
        }
        public function process_payment($order_id) {
            $order = wc_get_order($order_id);

            $args = array(
                "order_id" => $order_id,
                "amount" => $order->get_total(),
                "currency" => $this->get_option("invoice_currency"),
                "expiry" => $this->get_option("expiry"),
                "description" => str_replace("{{order_id}}", $order_id, $this->get_option("invoice_description")),
                "callback_url" => WC()->api_request_url("rodpay_lightning_webhook"),
                "return_url" => $this->get_return_url($order),
            );

            $response = wp_remote_post("https://api.rodpay.com/integration/woocommerce/invoices/", array(
                "body" => json_encode($args),
                "headers" => array(
                    "Content-Type" => "application/json",
                    "RodPay-Api-Key" => $this->get_option("api_key"),
                ),
            ));

            if (is_wp_error($response) || wp_remote_retrieve_response_code($response) !== 201) {
                wc_add_notice("Failed to create invoice. Please try again.", "error");
                return array(
                    "result" => "fail",
                );
            } else {
                $body = wp_remote_retrieve_body($response);
                $data = json_decode($body, true);
    
                $order->update_status("pending-payment", __("Awaiting Lightning payment.", "rodpay-lightning-for-woocommerce"));
                WC()->cart->empty_cart();
    
                return array(
                    "result" => "success",
                    "redirect" => $data["payment_url"],
                );
            }
        }
        public function payment_fields() {
            echo "<p>" . esc_html($this->description) . "</p>";
        }

        public function webhook() {
            $input = file_get_contents("php://input");
            $data = json_decode($input, true);
            if (! $data) {
                wp_die("Invalid JSON", 400);
            }

            $api_key = $this->get_option("api_key");
            if ($data["api_key"] !== $api_key) {
                wp_die("Invalid API Key", 403);
            }

            $order_id = $data["order_id"];
            $order = wc_get_order($order_id);
            if (! $order) {
                wp_die("Order not found", 404);
            }

            if ($order->get_status() !== "pending") {
                wp_die("Order already processed", 400);
            }

            $status = $data["status"];
            if ($status === "Settled") {
                $order->payment_complete();
                $order->reduce_order_stock();
            } else if ($status === "Expired") {
                $order->update_status("failed", __("Payment expired.", "rodpay-lightning-for-woocommerce"));
            }

            wp_die("OK", 200);

            return array(
                "result" => "success",
                "redirect" => $this->get_return_url($order),
            );
        }
    }

    function add_rodpay_lightning_gateway($methods) {
        $methods[] = "WC_Gateway_RodPay_Lightning";
        return $methods;
    }

    add_filter("woocommerce_payment_gateways", "add_rodpay_lightning_gateway");
}
